/**
 * @file datamux_ctrl.c
 *
 * @brief @{Source file containing the data MUX control functions for the
 * CCG firmware application.
 *
 * This implementation can be updated to support other mux and switch parts as
 * required.@}
 */

/*
 * Copyright (2014-2016), Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All rights reserved.
 *
 * This software, including source code, documentation and related materials
 * (“Software”), is owned by Cypress Semiconductor Corporation or one of its
 * subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
 * protection (United States and foreign), United States copyright laws and
 * international treaty provisions. Therefore, you may use this Software only
 * as provided in the license agreement accompanying the software package from
 * which you obtained this Software (“EULA”).
 *
 * If no EULA applies, Cypress hereby grants you a personal, nonexclusive,
 * non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Cypress’s integrated circuit
 * products. Any reproduction, modification, translation, compilation, or
 * representation of this Software except as specified above is prohibited
 * without the express written permission of Cypress. Disclaimer: THIS SOFTWARE
 * IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress reserves the
 * right to make changes to the Software without notice. Cypress does not
 * assume any liability arising out of the application or use of the Software
 * or any product or circuit described in the Software. Cypress does not
 * authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death (“High Risk Product”). By
 * including Cypress’s product in a High Risk Product, the manufacturer of such
 * system or application assumes all risk of such use and in doing so agrees to
 * indemnify Cypress against all liability.
 */

#include <pd.h>
#include <alt_modes_mngr.h>
#include <app.h>
#include <timer.h>
#include <dp_sid.h>

/* Update the data mux settings as required. */
bool mux_ctrl_set_cfg( uint8_t port, mux_select_t cfg, uint8_t polarity)
{
    /* Configure SBU-AUX switch based on CC polarity. */
    switch (cfg)
    {
        case MUX_CONFIG_ISOLATE:
        case MUX_CONFIG_SS_ONLY:
            /* Ensure that SBU-AUX switch is open. */
            sbu_switch_configure(SBU_NOT_CONNECTED, SBU_NOT_CONNECTED);
            /* Ensure that AUX1/2 don't have any resistor pull up/down. */
            aux_resistor_configure (AUX_NO_RESISTOR, AUX_NO_RESISTOR);
            break;

        case MUX_CONFIG_DP_4_LANE:
        case MUX_CONFIG_DP_2_LANE:
            if (polarity == 0)
            {
                /* CC1 is active. */
                sbu_switch_configure(SBU_CONNECT_AUX1, SBU_CONNECT_AUX2);
            }
            else
            {
                /* CC2 is active. */
                sbu_switch_configure(SBU_CONNECT_AUX2, SBU_CONNECT_AUX1);
            }
            
            /*
             * Configure AUX1/2 resistor pull up/down based on DP configuration.
             * If DP Config:
             * AUX1 : 470K pull down
             * AUX2 : 4.7M pull down
             *
             * If Non - DP Config:
             * AUX1 : 1M pull up
             * AUX2 : 1M pull down
             */
            if (dp_sink_get_pin_config () == DP_DFP_D_CONFIG_E)
            {
                aux_resistor_configure (AUX_1_470K_PD_RESISTOR, AUX_2_4P7MEG_PD_RESISTOR);
            }
            else
            {
                aux_resistor_configure (AUX_1_1MEG_PU_RESISTOR, AUX_2_1MEG_PD_RESISTOR);
            }
            break;

        default:
            /* No other configuration possible for C-DP 4-lane dongle. */
            break;
    }

     return true;
}

/* Initialize the MUX control SCB block. */
bool mux_ctrl_init (uint8_t port)
{
    /* Ensure SBU-AUX switch is open. */
    mux_ctrl_set_cfg(port, MUX_CONFIG_ISOLATE, 0);

    return true;
}

/* End of file */

